import os
import shutil

from . import log, run_shell


def concat(file_list, output, removed=False):
    """
    :param file_list: 合并的文件列表
    :param output: 输出文件
    :param removed: 合并成功之后是否删除原有的视频文件
    :return :
    """
    if len(file_list) < 2:
        shutil.copy(file_list[0], output)
    else:
        file_name_list = _get_file_name_list(file_list, output)
        concat_shell = (
            'ffmpeg '
            '-y -v info '
            '-f concat -safe 0 -i {} '
            '-c copy '
            '-bsf:a aac_adtstoasc -movflags +faststart '
            '{}'
        ).format(file_name_list, output)

        error_log = run_shell(concat_shell)
        if error_log:
            error_files = []
            for file_name in file_list:
                if file_name in error_log:
                    os.remove(file_name)
                    error_files.append(file_name)

            # TODO 合并的视频文件音频问题
            if error_files:
                sub_file_list = [file_name for file_name in file_list if file_name not in error_log]
                return concat(sub_file_list, output, removed=True)

        if os.path.isfile(file_name_list):
            os.remove(file_name_list)

    if os.path.isfile(output) and removed:
        for file in file_list:
            try:
                os.remove(file)
            except FileNotFoundError:
                pass

    return output


def _get_file_name_list(file_list, output):
    file_name_list = '{}.list.tmp'.format(output)
    with open(file_name_list, 'w+') as f:
        for file_name in file_list:
            if not os.path.isfile(file_name):
                log.warning('No such file or directory: %s', file_name)
                continue
            f.write('file {}\n'.format(file_name))
            log.debug('add file %s to %s', file_name, file_name_list)
    return file_name_list



import sys
import subprocess
import time

top_ = 'top -p {} -n 1 '
ps_aux_ = 'ps aux | grep {}'
# sudo apt install -y gawk
gawk_ = r''' gawk '{ if (NR == 8) { printf "%s\n", $0 } }' '''

ps_columns = ['user', 'pid', 'cpu', 'mem', 'vsz',
              'rss', 'tty', 'stat', 'start', 'time', 'command']

cpu_mean = {}


def get_cmd_result(proc_name):
    procs = []
    res = subprocess.run(ps_aux_.format(proc_name),
                         shell=True, stdout=subprocess.PIPE)
    for line in res.stdout.decode().split('\n'):
        proc = [column for column in line.split(' ') if column.strip()]
        if len(proc) < 10:
            continue
        if proc[10] in ['grep', '/bin/sh', 'bash']:
            continue
        commond = ' '.join(proc[10:])
        proc = proc[:10] + [commond]

        cpu_info = dict(zip(ps_columns, proc))
        procs.append(cpu_info)
    return procs


def run_print():
    if len(sys.argv) != 2:
        proc_name = input('process name > ')
    else:
        proc_name = sys.argv[1]
    while True:
        print('PID\tCPU\tMEM\tMCPU')
        for proc in get_cmd_result(proc_name):
            cmd = top_.format(proc['pid']) + '|' + gawk_
            s = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)
            for line in s.stdout.decode().split('\n'):

                line = line.replace('\x1b(B\x1b[m', '') \
                    .replace('\x1b[m\x0f', '') \
                    .split(' ')

                top = [column for column in line if column.strip()]
                if len(top) > 10:
                    print('\t'.join([top[0], top[8], top[9], proc['cpu'], proc['command']]))
        print()
        time.sleep(1)



import socket
import json
import urllib.request
import urllib.parse
from urllib.error import HTTPError
from .logger import log


def get_host_ip():
    try:
        ip = socket.gethostbyname(socket.gethostname())
    except Exception:
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        finally:
            s.close()
    if not ip:
        raise socket.gaierror(-2, 'Name or service not known')
    return ip


def do_request(method, url, data={},
               headers={'Content-Type': 'application/json'}):
    if method == 'GET' or method == 'DELETE':
        data = urllib.parse.urlencode(data)
        if data:
            url = url + '?' + data
        data = None
    else:
        data = json.dumps(data).encode('utf-8')

    req = urllib.request.Request(url, data, headers, method=method)
    try:
        response = urllib.request.urlopen(req)
        res_json = json.loads(response.read().decode('utf-8'))
    except HTTPError as e:
        log.error(e)
    except ValueError as e:
        log.error('JSONDecodeError.[{}]'.format(e))
    else:
        return res_json



import os
import time

from . import run_shell


def capture(stream_url, output_jpg, timeout=10, type='jpg'):
    """
    :param stream_url:
    :param output_jpg:
    :param timeout: 连接超时时间，单位s
    :param type: 图片类型，jpg、png
    :return : 截图成功与否
    """

    if type == 'jpg':
        _f = '-f mjpeg'
    else:
        _f = '-f image2'
    shell_cmd = (
        'ffmpeg '
        '-y -rw_timeout {} '  # 设置超时时间10s
        '-i {} '
        '-ss 0 '
        '{} '
        '-vframes 1 '
        '{}'
    ).format(timeout * 1000 * 1000, stream_url, _f, output_jpg)

    result = False

    # 尝试三次获取截图
    for _ in range(3):
        error_log = run_shell(shell_cmd, name='ffmpeg_capture')
        if not error_log:
            result = True
            break

        if os.path.isfile(output_jpg):
            os.remove(output_jpg)

        time.sleep(1)

    return result



import subprocess
import errno

try:
    from intelab_python_sdk.logger import log
except ImportError:
    import logging
    log = logging.getLogger('ffmpeg')
    logging.basicConfig(format='%(asctime)s - %(pathname)s[line:%(lineno)d] - %(levelname)s: %(message)s',
                        level=logging.DEBUG)


def run_shell(cmd, name='ffmpeg'):
    """
    运行ffmpeg shell
    :param cmd:
    :param name:
    :return:
    """
    try:
        log.info(cmd)
        ffmpeg_proc = subprocess.Popen(cmd, shell=True,
                                       close_fds=True,
                                       stdin=subprocess.PIPE,
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.STDOUT, bufsize=1)
    except OSError as e:
        if e.errno == errno.ENOENT:
            raise "Executable '{0}' not found".format('ffmpeg')
        else:
            raise

    # 实时打印ffmpeg日志
    return_code = ffmpeg_proc.poll()
    log_buffer = error_log = ''

    while return_code is None:
        # 这里会阻塞,所以不用获取所有的日志才返回，获取81个字节就可以返回了！
        line = ffmpeg_proc.stdout.readline(81)
        log_buffer += line.decode('utf-8', 'ignore')

        if r'\n' in str(line):
            if 'Impossible to open' in log_buffer:
                error_log += log_buffer
            if 'block unavailable' in log_buffer:
                error_log += log_buffer
            if 'Cannot open connection' in log_buffer:
                error_log += log_buffer
            log.debug('%s:%s', name, log_buffer.strip())
            log_buffer = ''

        return_code = ffmpeg_proc.poll()

    if log_buffer:
        if 'Impossible to open' in log_buffer:
            error_log += log_buffer
        log.debug('%s:%s', name, log_buffer.strip())
    return error_log


def _print_log(name, log_buffer, error_log):
    if 'Impossible to open' in log_buffer:
        error_log += log_buffer
    log.debug('%s:%s', name, log_buffer.strip())
    return error_log



import os
import pickle


class FileSystemCache(object):
    """ 文件缓存

    :param chche_dir: 缓存文件目录

    Usage::

        >>> from intelab_python_sdk.cache import FileSystemCache
        >>> from datetime import datetime
        >>> cache = FileSystemCache('.cache')
        >>> cache.set('12', datetime.now())
        >>> cache.get('12')
        datetime.datetime(2019, 12, 19, 12, 16, 1, 511077)
        >>> list(cache.keys())
        ['12']
        >>> cache.delete('12')

    """

    def __init__(self, cache_dir):
        self.base_dir = cache_dir
        if not os.path.exists(self.base_dir):
            os.makedirs(self.base_dir)

    def _get_filename(self, key):
        return os.path.join(self.base_dir, key)

    def get(self, key, default=None):
        filename = self._get_filename(key)
        try:
            with open(filename, 'rb') as f:
                value = pickle.load(f)
        except (IOError, OSError, EOFError, pickle.PickleError):
            value = None
        if value is None and default is not None:
            value = default

        return value

    def set(self, key, value):
        filename = self._get_filename(key)
        with open(filename, 'wb') as f:
            pickle.dump(value, f, pickle.HIGHEST_PROTOCOL)

    def delete(self, key):
        filename = self._get_filename(key)
        if os.path.isfile(filename):
            os.remove(filename)

    def keys(self):
        keys = []
        for root, dirs, files in os.walk(self.base_dir, topdown=False):
            for file in files:
                keys.append(os.path.split(file)[-1])

        return iter(keys)



from . import run_shell


def ffmpeg_mp4tomp3(mp4_file, output_file):
    """
    mp4视频文件转mp3音频文件
    """
    shell_cmd = (
        'ffmpeg '
        '-y '
        '-i {} '
        '-vn '
        '-f mp3 '
        '{}'
    ).format(mp4_file, output_file)
    run_shell(shell_cmd, name='ffmpeg_mp4tomp3')
    return output_file



from codecs import open
from setuptools import setup, find_packages


with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()

setup(
    name='intelab_python_sdk',
    version='0.6.16',
    url='https://github.com/zwwangoo/intelab-python-sdk',
    long_description=readme,
    long_description_content_type='text/x-rst',
    author='zwwangoo',
    author_email='w_angzhiwen@163.com',
    packages=find_packages(exclude=('tests', 'tests.*')),
    zip_safe=False,
    include_package_data=True,
    python_requires='>=3.5',
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    entry_points={
        'console_scripts': [
            'intelab_python_sdk = intelab_python_sdk.helper.cpu:run_print'
        ]
    }

)



import os
import re
import threading
import subprocess

from datetime import datetime

from . import log


class FfmpegRecordThread(threading.Thread):

    def __init__(self, name, stream_url, out_file_path, video_duration=300, timeout=5000000, **kwargs):
        """
        :param name:
        :param stream_url: 流地址
        :param out_file_path:
        :param video_duration: 视频持续时间，默认持续 5 * 60 秒
        :param timeout: 超时断开连接，设定时间，默认5s,单位为微妙级
        :param kwargs:
        """
        sub_shell = ''
        if 'rtsp_transport' in kwargs:
            sub_shell = '-rtsp_transport {} '.format(kwargs.pop('rtsp_transport'))
        if 'stimeout' in kwargs:
            sub_shell += '-stimeout {} '.format(kwargs.pop('stimeout'))
        else:
            sub_shell += '-rw_timeout {} '.format(timeout)

        super(FfmpegRecordThread, self).__init__(**kwargs)

        self.create_time = datetime.now()
        self.name = name
        self.stream_url = stream_url
        self.out_file_path = out_file_path
        self.out_file_dir = os.path.split(out_file_path)[0]
        self.video_duration = video_duration
        self.file_name_out_list = os.path.join(self.out_file_dir, name + '.txt')

        self.shell_cmd_mp4 = (
            'ffmpeg '
            '-y '                                     # 覆盖输出文件
            '-v info '
            '-rtbufsize 1m '
            '{sub_shell}'                                      # 超时断开连接，设定是5s
            '-i "{stream_url}" '                        # 输入视频文件或流等其他
            '-movflags faststart+frag_keyframe '      # 使mp4支持渐进式下载
            '-c:v copy '                              # 原始编解码数据必须被拷贝
            '-c:a copy '                              # 设定声音编码，降低CPU使用
            '-f segment '                             # 输出流切片
            '-segment_format mp4 '                    # 流输出格式
            '-strftime 1 '                            # 设置切片名为生成切片的时间点
            '-segment_time {segment_time} '           # 流切分时长
            '-reset_timestamps 1 '                    # 每个切片都重新初始化时间戳
            '-segment_list "{file_name_out_list}" '     # 切片列表主文件名，输入是在文件写入完成之后
            '-segment_list_size 10 '                  # 列表文件长度
            '-segment_list_entry_prefix "{out_path}" '  # 写文件列表时写入每个切片路径的前置路径

            '"{out_file}" '                             # 输出文件名
        ).format(sub_shell=sub_shell,
                 stream_url=self.stream_url,
                 segment_time=video_duration,
                 file_name_out_list=self.file_name_out_list,
                 out_path=self.out_file_dir + '/',
                 out_file=self.out_file_path)

        os.makedirs(self.out_file_dir, exist_ok=True)
        self.ffmpeg_proc = None
        self.video_resolution = None

    def kill(self):
        try:
            self.ffmpeg_proc.communicate(input='q'.encode(), timeout=2)
        except Exception as e:
            log.debug(e)
            self.ffmpeg_proc.kill()

    def run(self):
        log.debug(self.shell_cmd_mp4)
        self.ffmpeg_proc = subprocess.Popen(
            self.shell_cmd_mp4, shell=True,
            close_fds=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=1)

        # 实时打印ffmpeg日志
        log_buffer = ''
        return_code = self.ffmpeg_proc.poll()
        while return_code is None:
            # 这里会阻塞,所以不用获取所有的日志才返回，获取81个字节就可以返回了！
            line = self.ffmpeg_proc.stdout.readline(81)
            log_buffer += line.decode()
            # 换行打印日志
            if r'\n' in str(line) or r'\r' in str(line):
                if 'Packet mismatch' in log_buffer:
                    self.kill()
                    break

                # 处理日志中\r的超过81字符的日志长度
                log_buffer = log_buffer.split('\r')
                if len(log_buffer) > 1:
                    log_buffer, stdout = '\n'.join(log_buffer[:-1]), log_buffer[-1]
                else:
                    log_buffer = ''.join(log_buffer)
                    stdout = ''

                # 通过日志获取分辨率
                resolution = re.findall(r'(\d{3,4}x\d{3,4}),', log_buffer)
                if resolution:
                    self.video_resolution = resolution[0]

                log.debug('%s:%s', self.name, log_buffer.strip())
                log_buffer = stdout

            return_code = self.ffmpeg_proc.poll()
        log.debug('%s:%s', self.name, log_buffer.strip())

        self.ffmpeg_proc.kill()
        log.info('{}的ffmpeg exit.'.format(self.name))



from . import log, run_shell


def prune(file_name, output, start_time=0, duration=None):
    """
    :param file_name: 剪切的文件
    :param output: 输出文件
    :param start_time: 剪切开始的时间
    :param duration: 持续时间
    :return :
    """

    if duration:
        _duration = '-t {} '.format(duration)
    else:
        _duration = ''

    shell = (
        'ffmpeg '
        '-y '
        '-ss {} {}'
        '-i {} '
        '-c:a copy '
        '-c:v copy '
        '{}'
    ).format(start_time, _duration, file_name, output)
    log.info(shell)
    run_shell(shell)

    return output
